package org.firstinspires.ftc.teamcode.subsystem;

import static org.firstinspires.ftc.teamcode.configs.RobotConstants.liftPower;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.liftToFloatPos;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.liftToFloorPos;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.liftToHighBucketPos;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.liftToLowBucketPos;

import com.qualcomm.robotcore.hardware.DcMotor;
import com.qualcomm.robotcore.hardware.DcMotorSimple;
import com.qualcomm.robotcore.hardware.HardwareMap;

import org.firstinspires.ftc.teamcode.util.action.RunAction;

public class LiftSubsystem {

    public DcMotor lift;
    public RunAction toFloor, toLowBucket, toHighBucket;

    public enum LiftState {
        FLOOR, LOW_BUCKET, HIGH_BUCKET, FLOAT
    }

    private LiftState liftState;

    public LiftSubsystem(HardwareMap hardwareMap) {
        lift = hardwareMap.get(DcMotor.class, "lift-motor");
        toFloor = new RunAction(this::toFloor);
        toLowBucket = new RunAction(this::toLowBucket);
        toHighBucket = new RunAction(this::toHighBucket);
    }

    public void setTarget(int b) {
        lift.setTargetPosition(b);
        lift.setMode(DcMotor.RunMode.RUN_TO_POSITION);
    }

    public void switchState() {
        if (this.liftState == LiftState.FLOOR) {
            this.toFloor();
        } else if (this.liftState == LiftState.FLOAT) {
            this.toLowBucket();
        } else if (this.liftState == LiftState.LOW_BUCKET) {
            this.toHighBucket();
        } else if (this.liftState == LiftState.HIGH_BUCKET) {
            this.toFloor();
        }
    }

    public void toFloor() {
        this.setTarget(liftToFloorPos);
        this.setState(LiftState.FLOOR);
    }

    public void toFloat() {
        this.setTarget(liftToFloatPos);
        this.setState(LiftState.FLOAT);
    }

    public void toLowBucket() {
        this.setTarget(liftToLowBucketPos);
        this.setState(LiftState.LOW_BUCKET);
    }

    public void toHighBucket() {
        this.setTarget(liftToHighBucketPos);
        this.setState(LiftState.HIGH_BUCKET);
    }

    public void init() {
        lift.setMode(DcMotor.RunMode.STOP_AND_RESET_ENCODER);
        lift.setDirection(DcMotorSimple.Direction.REVERSE);
        lift.setPower(liftPower);
    }

    private void setState(LiftState liftState) {
        this.liftState = liftState;
    }

    public int getPosition() {
        return lift.getCurrentPosition();
    }

    public void start() {
        this.toFloor();
    }

}
